const dbus = require('dbus-next');

class SwaiRegistry {
    constructor() {
        this.bus = null;
        this.proxy = null;
        this.available = false;
        this.registeredTitles = new Set();
        this.initializeDBus();
    }

    async initializeDBus() {
        try {
            this.bus = dbus.sessionBus();
            
            // Check if the Mutter SwaiRegistry interface is available
            await this.checkInterfaceAvailability();
            
            if (this.available) {
                console.log('Mutter SwaiRegistry interface is available');
                this.proxy = await this.bus.getProxyObject('org.gnome.Mutter', '/org/gnome/Mutter/SwaiRegistry');
                this.swaiRegistry = this.proxy.getInterface('org.gnome.Mutter.SwaiRegistry');
            } else {
                console.log('Mutter SwaiRegistry interface not available, WM_CLASS customization disabled');
            }
        } catch (error) {
            console.warn('Failed to initialize DBus connection for SwaiRegistry:', error.message);
            this.available = false;
        }
    }

    async checkInterfaceAvailability() {
        try {
            // Try to get the proxy object to check if the interface exists
            const proxy = await this.bus.getProxyObject('org.gnome.Mutter', '/org/gnome/Mutter/SwaiRegistry');
            const introspection = await proxy.Introspect();
            
            // Check if the SwaiRegistry interface is in the introspection data
            if (introspection.includes('org.gnome.Mutter.SwaiRegistry')) {
                this.available = true;
            } else {
                this.available = false;
            }
        } catch (error) {
            // If we can't connect or introspect, the interface isn't available
            this.available = false;
        }
    }

    async registerTitle(internalTitle, displayTitle) {
        if (!this.available || !this.swaiRegistry) {
            console.log(`Cannot register title ${internalTitle} - DBus interface not available`);
            return false;
        }

        try {
            await this.swaiRegistry.RegisterTitle(internalTitle, displayTitle);
            this.registeredTitles.add(internalTitle);
            console.log(`Registered title: ${internalTitle} -> ${displayTitle}`);
            return true;
        } catch (error) {
            console.error(`Failed to register title ${internalTitle}:`, error.message);
            return false;
        }
    }

    async updateTitle(internalTitle, displayTitle) {
        // For the Mutter patch, updating is the same as registering
        // (RegisterTitle can be called multiple times for the same internal title)
        return await this.registerTitle(internalTitle, displayTitle);
    }

    async unregisterTitle(internalTitle) {
        if (!this.available || !this.swaiRegistry) {
            return false;
        }

        try {
            await this.swaiRegistry.UnregisterTitle(internalTitle);
            this.registeredTitles.delete(internalTitle);
            console.log(`Unregistered title: ${internalTitle}`);
            return true;
        } catch (error) {
            console.error(`Failed to unregister title ${internalTitle}:`, error.message);
            return false;
        }
    }

    async cleanup() {
        if (!this.available || !this.swaiRegistry) {
            return;
        }

        try {
            // Unregister all registered titles
            const promises = Array.from(this.registeredTitles).map(title => 
                this.unregisterTitle(title)
            );
            await Promise.all(promises);
            
            if (this.bus) {
                this.bus.disconnect();
            }
        } catch (error) {
            console.warn('Error during SwaiRegistry cleanup:', error.message);
        }
    }

    isAvailable() {
        return this.available;
    }

    /**
     * Generate a unique internal title for window registration
     * This will be set as the actual window title in Electron
     */
    generateInternalTitle(appId, windowId) {
        return `io.stillhq.swai.${appId}`;
    }
}

module.exports = SwaiRegistry; 