const {
    BaseWindow,
    BrowserWindow,
    WebContentsView,
    session,
    ipcMain,
} = require("electron");
const fs = require("fs");
const path = require("path");
const os = require("os");
const { parse_arg, openUrlInBrowser, wildCardMatchList } = require("../utils/swai_utils");
const Ecosystem = require("../utils/Ecosystem");

class WindowManager {
    constructor(debug_mode = false, ecosystemId = null, swaiRegistry = null) {
        this.windowId = 0;
        this.debug_mode = debug_mode;
        this.ecosystemId = ecosystemId;
        this.swaiRegistry = swaiRegistry;
    }

    createWindow(webapp) {
        this.windowId += 1;
        let boundsTimeout = null;
        let webapp_ecosystem = null;

        // Use ecosystem ID for data path so apps in same ecosystem share data/cookies
        const dataId = this.ecosystemId || webapp.app_id;
        let data_path = path.join(os.homedir(), '.local', 'share', 'swai', 'app_data', dataId);
        const { app } = require("electron");
        app.setPath("userData", path.join(data_path, "userData"));
        app.setPath("appData", path.join(data_path, "appData"));
        app.setPath("sessionData", path.join(data_path, "sessionData"));
        app.setPath("cache", path.join(data_path, "cache"));
        app.setPath("logs", path.join(data_path, "logs"));
        app.setPath("crashDumps", path.join(data_path, "crashDumps"));

        // Use ecosystem ID for session partition so apps in same ecosystem share sessions/cookies
        let webAppSession = session.fromPartition(`persist:${dataId}`);

        let thisWindow = null;

        // For ecosystem apps, use internal title for DBus registration
        let windowTitle = webapp.app_name;
        let internalTitle = null;
        let currentDisplayTitle = webapp.app_name; // Track current display title to avoid redundant DBus calls
        
        if (webapp.ecosystem && this.swaiRegistry && this.swaiRegistry.isAvailable()) {
            internalTitle = this.swaiRegistry.generateInternalTitle(webapp.app_id, this.windowId);
            windowTitle = internalTitle; // Set window title to internal title for DBus registration
        }

        thisWindow = new BaseWindow({
            width: 1200,
            height: 900,
            title: windowTitle,
            frame: false,
        });

        const set_title = (title) => {
            if (internalTitle && this.swaiRegistry && this.swaiRegistry.isAvailable()) {
                // For ecosystem apps, keep window title as internal title but update DBus display title
                thisWindow.title = internalTitle;
                
                // Only update DBus if the display title has actually changed
                if (title !== currentDisplayTitle) {
                    currentDisplayTitle = title;
                    this.swaiRegistry.updateTitle(internalTitle, title).catch(error => {
                        console.warn('Failed to update DBus title registration:', error.message);
                    });
                }
            } else {
                // For non-ecosystem apps, set title normally
                thisWindow.title = title;
            }
        };

        let titlebar = new WebContentsView({
            webPreferences: {
                contextIsolation: false,
                nodeIntegration: true,
                additionalArguments: [
                    `--windowId=${this.windowId}`,
                    `--webappName=${webapp.app_name}`,
                ],
            }
        });

        let mainWebContents = new WebContentsView({
            nodeIntegration: false,
            contextIsolation: true,
            webPreferences: {
                session: webAppSession
            }
        });

        titlebar.webContents.loadURL(
            `file://${path.join(__dirname, "../components/titlebar.html")}`,
        );
        thisWindow.removeMenu();
        thisWindow.contentView.addChildView(titlebar);
        thisWindow.contentView.addChildView(mainWebContents);

        let custom_main_url = parse_arg("--custom_main_url");
        if (custom_main_url) {
            webapp.main_url = custom_main_url;
            if (!wildCardMatchList(custom_main_url, webapp.allowed_urls)) {
                webapp.allowed_urls.push(custom_main_url);
            }
        }

        mainWebContents.webContents.loadURL(webapp.main_url);

        if (this.debug_mode) {
            titlebar.webContents.openDevTools();
            mainWebContents.webContents.openDevTools();
        }

        const updateBounds = () => {
            const { width, height } = thisWindow.getBounds();
            let titlebarHeight = this.debug_mode ? 300 : 36;
            titlebar.setBounds({ x: 0, y: 0, width, height: titlebarHeight });
            mainWebContents.setBounds({
                x: 0,
                y: titlebarHeight,
                width,
                height: height - titlebarHeight,
            });
        };

        updateBounds();

        thisWindow.on("resize", () => {
            updateBounds();
            if (boundsTimeout) {
                clearTimeout(boundsTimeout);
            }
            boundsTimeout = setTimeout(updateBounds, 1000);
        });

        this._setupMainWebContents(mainWebContents, thisWindow, webapp, set_title, webAppSession, titlebar);
        this._setupIpcHandlers(thisWindow, mainWebContents);
        this._setupContextMenu(mainWebContents);

        // Register the window with DBus for custom WM_CLASS handling (ecosystem apps only)
        if (internalTitle && this.swaiRegistry) {
            this.swaiRegistry.registerTitle(internalTitle, currentDisplayTitle).then(success => {
                if (success) {
                    console.log(`Registered ${webapp.app_name} with custom WM_CLASS: ${webapp.app_id}`);
                }
            }).catch(error => {
                console.warn('Failed to register title with DBus:', error.message);
            });

            // Setup cleanup when window is closed
            thisWindow.on('closed', () => {
                if (this.swaiRegistry) {
                    this.swaiRegistry.unregisterTitle(internalTitle).catch(error => {
                        console.warn('Failed to unregister title:', error.message);
                    });
                }
            });
        }

        return thisWindow;
    }

    _setupMainWebContents(mainWebContents, thisWindow, webapp, set_title, webAppSession, titlebar) {
        mainWebContents.webContents.on("dom-ready", () => {
            console.log("dom ready");

            if (thisWindow) {
                const cssPath = path.join(__dirname, "../../system_theme.css");
                fs.readFile(cssPath, "utf-8", (err, data) => {
                    if (err) {
                        console.error("Failed to read CSS file:", err);
                        return;
                    }
                    if (thisWindow) {
                        mainWebContents.webContents.insertCSS(data).catch((error) => {
                            console.error("Failed to inject CSS:", error);
                        });
                    }
                });
            }

            mainWebContents.webContents.setWindowOpenHandler((details) => {
                console.log("Attempted new window");
                if (!this._get_navigation_allowed(
                    mainWebContents.webContents.getURL(),
                    details.url,
                    webapp
                )) {
                    openUrlInBrowser(details.url);
                    return { action: "deny" };
                }
                if (!webapp.allow_multi_window) {
                    this.createTransientWindow(details.url, webAppSession, thisWindow);
                    return { action: "deny" };
                }
                if (wildCardMatchList(details.url, webapp.allowed_urls)) {
                    let new_window_app = webapp.clone();
                    new_window_app.main_url = details.url;
                    this.createWindow(new_window_app);
                    return { action: "deny" };
                } else {
                    this.createTransientWindow(details.url, webAppSession, thisWindow);
                    return { action: "deny" };
                }
                return { action: "allow" };
            });

            this._setupNavigationHandlers(mainWebContents, webapp, set_title, titlebar);
            this._setupKeyboardShortcuts(mainWebContents);
        });
    }

    _setupNavigationHandlers(mainWebContents, webapp, set_title, titlebar) {
        mainWebContents.webContents.on("will-navigate", (event, navigationUrl) => {
            if (!this._get_navigation_allowed(
                mainWebContents.webContents.getURL(),
                navigationUrl,
                webapp
            )) {
                event.preventDefault();
                openUrlInBrowser(navigationUrl);
            }
        });

        mainWebContents.webContents.on("did-navigate", (event, navigationUrl) => {
            set_title(mainWebContents.webContents.getTitle());
            this._sendUrlChanged(mainWebContents, titlebar);
        });

        mainWebContents.webContents.on("did-navigate-in-page", (event, navigationUrl) => {
            set_title(mainWebContents.webContents.getTitle());
            this._sendUrlChanged(mainWebContents, titlebar);
        });

        mainWebContents.webContents.on("page-title-updated", (event, title) => {
            set_title(mainWebContents.webContents.getTitle());
            this._sendUrlChanged(mainWebContents, titlebar);
        });
    }

    _sendUrlChanged(mainWebContents, titlebar) {
        if (titlebar) {
            titlebar.webContents.send(
                `url-changed-${this.windowId}`,
                mainWebContents.webContents.getTitle(),
                mainWebContents.webContents.navigationHistory.canGoBack(),
                mainWebContents.webContents.navigationHistory.canGoForward(),
            );
        }
    }

    _setupKeyboardShortcuts(mainWebContents) {
        mainWebContents.webContents.on("before-input-event", (event, input) => {
            if (input.control) {
                if (input.key === "I" && input.control && input.shift) {
                    mainWebContents.webContents.toggleDevTools();
                    event.preventDefault();
                } else if (input.key === "+") {
                    mainWebContents.webContents.setZoomFactor(
                        mainWebContents.webContents.getZoomFactor() + 0.1,
                    );
                    event.preventDefault();
                } else if (input.key === "-") {
                    mainWebContents.webContents.setZoomFactor(
                        mainWebContents.webContents.getZoomFactor() - 0.1,
                    );
                    event.preventDefault();
                } else if (input.key === "0") {
                    mainWebContents.webContents.setZoomFactor(1.0);
                    event.preventDefault();
                } else if (input.key == "r") {
                    mainWebContents.webContents.reload();
                    event.preventDefault();
                }
            }
        });
    }

    _setupIpcHandlers(thisWindow, mainWebContents) {
        ipcMain.on(`back-${this.windowId}`, () => {
            if (thisWindow) {
                mainWebContents.webContents.navigationHistory.goBack();
            }
        });

        ipcMain.on(`forward-${this.windowId}`, () => {
            if (thisWindow) {
                mainWebContents.webContents.navigationHistory.goForward();
            }
        });

        ipcMain.on(`close-${this.windowId}`, () => {
            if (thisWindow) {
                thisWindow.close();
            }
        });

        ipcMain.on(`minimize-${this.windowId}`, () => {
            if (thisWindow) {
                thisWindow.minimize();
            }
        });

        ipcMain.on(`maximize-${this.windowId}`, () => {
            if (thisWindow) {
                if (thisWindow.isMaximized()) {
                    thisWindow.unmaximize();
                } else {
                    thisWindow.maximize();
                }
            }
        });
    }

    async _setupContextMenu(mainWebContents) {
        try {
            const contextMenu = await import("electron-context-menu");
            const dispose = contextMenu.default({
                window: mainWebContents.webContents,
                showInspectElement: false,
            });
        } catch (error) {
            console.error("Failed to setup context menu:", error);
        }
    }

    _get_navigation_allowed(currentUrl, navigationUrl, webapp) {
        if (wildCardMatchList(navigationUrl, webapp.not_allowed_urls)) {
            console.log("Opening browser window");
            return false;
        }
        if (wildCardMatchList(currentUrl, webapp.login_urls)) {
            console.log("Bypassed Browser Window1");
            return true;
        }
        if (!wildCardMatchList(currentUrl, webapp.allowed_urls)) {
            console.log("Bypassed Browser Window2");
            return true;
        }
        if (wildCardMatchList(navigationUrl, webapp.allowed_urls)) {
            return true;
        }
        return false;
    }

    createTransientWindow(url, session, parent) {
        let transientWindow = new BrowserWindow({
            width: 800,
            height: 600,
            parent: parent,
            modal: true,
            title: "SWAI Web App",
            webPreferences: {
                session: session,
            },
        });
        transientWindow.loadURL(url);
        transientWindow.once("ready-to-show", () => {
            transientWindow.show();
        });
        transientWindow.removeMenu();
    }
}

module.exports = WindowManager; 